#ifndef __PACKET_HEADER_INCLUDED__
#define __PACKET_HEADER_INCLUDED__

#include <g_globals.h>
#include <capdev.h>

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#ifdef HAVE_NETINET_IN_SYSTM_H
#include <netinet/in_systm.h>
#endif

#ifdef HAVE_NETINET_TCP_H
#define __FAVOR_BSD
#include <netinet/tcp.h>
#undef __FAVOR_BSD
#endif

#ifdef NETINET_IF_ETHER_H
#include <netinet/if_ether.h>
#endif

#ifdef HAVE_NETINET_UDP_H
#include <netinet/udp.h>
#endif

#ifdef HAVE_NETINET_IP_H
#include <netinet/ip.h>
#endif

#ifdef HAVE_NETINET_IP_ICMP_H
#include <netinet/ip_icmp.h>
#endif

#ifdef HAVE_NETINET_IGMP_H
#include <netinet/igmp.h>
#endif

#ifdef HAVE_NETINET_IP6_H
#include <netinet/ip6.h>
#endif

#define PORT ushort
#define CAP_SNAPLEN 4096

/* From the linux kernel */
struct ethhdr{
	unsigned char   h_dest[6];       /* destination eth addr */
	unsigned char   h_source[6];     /* source ether addr    */
	unsigned short  h_proto;         /* packet type ID field */
};

typedef enum network_layer{
	LAYER_LINK,
	LAYER_NET,
	LAYER_TRANS,
	LAYER_APP,
	LAYER_MAX
}network_layer_t;

typedef struct layer {
	uint id;
	char *name;
	union {
		void			*raw;
		struct tcphdr  		*tcp;
		struct udphdr		*udp;
		struct icmphdr		*icmp;
		struct iphdr		*ip;
		struct ethhdr		*mac;
	}h;
}layer_t, *p_layer;

typedef struct packet {
	time_t		time;	/* Timestamp */
	struct tm	*ltime;	/* Localised timestamp */
	
	uint		caplen;	/* Length of present data */
	uint		len;	/* Real length of data on the wire */

	p_capdev	capdev; /* The capture device that cought it */
	
	PORT		sport, dport; /* Source and dest ports */

	layer_t         layer[LAYER_MAX]; /* Network layers */
	uchar		data[CAP_SNAPLEN]; /* Full packet data */
}packet_t, *p_packet;

#endif //__PACKET_HEADER_INCLUDED__
